@@ -1,5 +1,5 @@
 # This file is part of caucase
-# Copyright (C) 2017-2020  Nexedi SA
+# Copyright (C) 2017-2021  Nexedi SA
 #     Alain Takoudjou <alain.takoudjou@nexedi.com>
 #     Vincent Pelletier <vincent@nexedi.com>
 #
@@ -641,13 +641,12 @@ def main(argv=None, stdout=sys.stdout, stderr=sys.stderr):
       stdout=stdout,
       stderr=stderr,
     ) as client:
-      ca_list = [
-        utils.load_ca_certificate(x)
-        for x in utils.getCertList({
+      ca_list = utils.load_valid_ca_certificate_list(
+        ca_pem_list=utils.getCertList({
           MODE_SERVICE: args.ca_crt,
           MODE_USER: args.user_ca_crt,
-        }[args.mode])
-      ]
+        }[args.mode]),
+      )
       client.putCSR(args.send_csr)
       client.getCSR(args.get_csr)
       warning, error = client.getCRT(warning, error, args.get_crt, ca_list)
@@ -672,16 +671,22 @@ def main(argv=None, stdout=sys.stdout, stderr=sys.stderr):
       if args.list_csr:
         client.listCSR(args.mode)
     # update our CRL after all revocations we were requested
-    updated |= CaucaseClient.updateCRLFile(cas_url, args.crl, [
-      utils.load_ca_certificate(x)
-      for x in utils.getCertList(args.ca_crt)
-    ])
+    updated |= CaucaseClient.updateCRLFile(
+      cas_url,
+      args.crl,
+      utils.load_valid_ca_certificate_list(
+        ca_pem_list=utils.getCertList(args.ca_crt),
+      ),
+    )
     # --update-user, CRL part
     if args.update_user:
-      updated |= CaucaseClient.updateCRLFile(cau_url, args.user_crl, [
-        utils.load_ca_certificate(x)
-        for x in utils.getCertList(args.user_ca_crt)
-      ])
+      updated |= CaucaseClient.updateCRLFile(
+        cau_url,
+        args.user_crl,
+        utils.load_valid_ca_certificate_list(
+          ca_pem_list=utils.getCertList(args.user_ca_crt),
+        ),
+      )
     finished = True
   finally:
     if updated and args.on_renew:
@@ -922,20 +927,19 @@ def updater(argv=None, until=utils.until):
         # Note: CRL expiration should happen several time during CA renewal
         # period, so it should not be needed to keep track of CA expiration
         # for next deadline.
-      ca_crt_list = [
-        utils.load_ca_certificate(x)
-        for x in utils.getCertList(args.ca)
-      ]
+      ca_crt_list = utils.load_valid_ca_certificate_list(
+        ca_pem_list=utils.getCertList(args.ca),
+      )
       if RetryingCaucaseClient.updateCRLFile(ca_url, args.crl, ca_crt_list):
         print('Got new CRL')
         updated = True
-      for crl_pem in utils.getCRLList(args.crl):
+      for _, crl in utils.iter_valid_crl_list(
+        crl_pem_list=utils.getCRLList(args.crl),
+        trusted_cert_list=ca_crt_list,
+      ):
         next_deadline = min(
           next_deadline,
-          utils.load_crl(
-            crl_pem,
-            ca_crt_list,
-          ).next_update - crl_threshold,
+          crl.next_update - crl_threshold,
         )
       if args.crt:
         crt_pem, key_pem, key_path = utils.getKeyPair(args.crt, args.key)
