@@ -1,5 +1,5 @@
 # This file is part of caucase
-# Copyright (C) 2017-2020  Nexedi SA
+# Copyright (C) 2017-2021  Nexedi SA
 #     Alain Takoudjou <alain.takoudjou@nexedi.com>
 #     Vincent Pelletier <vincent@nexedi.com>
 #
@@ -513,6 +513,34 @@ def load_ca_certificate(data):
   _verifyCertificateChain(crt, [crt], None)
   return crt
 
+def iter_valid_ca_certificate_list(ca_pem_list):
+  """
+  Load multiple CA certificates from a list of PEM-encoded values.
+
+  Yields the PEM-encoded value along with the loaded CA.
+
+  Skips items failing to load.
+  """
+  for ca_pem in ca_pem_list:
+    try:
+      ca = load_ca_certificate(ca_pem)
+    except CertificateVerificationError:
+      continue
+    else:
+      yield (ca_pem, ca)
+
+def load_valid_ca_certificate_list(ca_pem_list):
+  """
+  Load CA certificates from PEM-encoded data, skipping items which fail
+  signature verification.
+
+  Returns the list of loaded CA certificates.
+  """
+  return [
+    ca
+    for _, ca in iter_valid_ca_certificate_list(ca_pem_list)
+  ]
+
 def load_certificate(data, trusted_cert_list, crl_list):
   """
   Load a certificate from PEM-encoded data.
@@ -591,6 +619,22 @@ def load_crl(data, trusted_cert_list):
       return crl
   raise cryptography.exceptions.InvalidSignature
 
+def iter_valid_crl_list(crl_pem_list, trusted_cert_list):
+  """
+  Load multiple certificate revocation lists from a list of PEM-encoded values.
+
+  Yields the PEM-encoded value along with the loaded CRL.
+
+  Skips items failing to load.
+  """
+  for crl_pem in crl_pem_list:
+    try:
+      crl = load_crl(crl_pem, trusted_cert_list)
+    except cryptography.exceptions.InvalidSignature:
+      continue
+    else:
+      yield (crl_pem, crl)
+
 def _getAuthorityKeyIdentifier(cert):
   return cert.extensions.get_extension_for_class(
     x509.AuthorityKeyIdentifier,
